from meapp.api.response import ApiResp,ApiState
from meapp.api.crawl import param_crawl
from rest_framework.viewsets import ViewSet
from rest_framework.decorators import action
from meapp.core.models import CodeRecord

class SmsView(ViewSet):
    @action(methods=['post', 'options', 'get'], detail=False)
    @param_crawl(checker={
        'mobile': {'n': True},
        'type': {'n': True}, # login | register
    })
    def sendCode(self,request):
        pass




from rest_framework import viewsets
from rest_framework.exceptions import APIException
from rest_framework.decorators import action
from rest_framework.authtoken.views import ObtainAuthToken
from rest_framework.authtoken.models import Token


from ..response import ApiResponse,ApiState,ApiResp
from ..crawl import param_crawl




class ApiAuthView(viewsets.ViewSetMixin,ObtainAuthToken):

    def user_can_authenticate(self, user):
        """
        Reject users with is_active=False. Custom user models that don't have
        that attribute are allowed.
        """
        is_active = getattr(user, 'is_active', None)
        return is_active or is_active is None

    def authenticate(self, request, username=None, password=None, **kwargs):
        if username is None:
            print('11100')
            username = kwargs.get(self.UserModel.USERNAME_FIELD)
            print('11100---1')

        try:
            user = self.UserModel._default_manager.get_by_natural_key(username)
        except self.UserModel.DoesNotExist:
            # Run the default password hasher once to reduce the timing
            # difference between an existing and a nonexistent user (#20760).
            self.UserModel().set_password(password)
            raise APIException('用户名或密码错误')
        else:
            if user.check_password(password) and self.user_can_authenticate(user):
                return user
            else:
                return None

    Token = None
    @property
    def model(self):
        return self.Token.model

    @property
    def UserModel(self):
        return self.Token.model


    def loginResponse(self,user,token):
        data = {'id':user.id}
        data['token'] = token
        return data

    def registResponse(self,user,token):
        return {'id': user.id, 'token': token}

    @action(methods=['post','options','get'], detail=False)
    def register(self, request):
        if self.model is None:
            return ApiResponse(ApiState.exception, "login error: ApiAuthView can not be None (authModel inherit from User model class )", "")
        model = self.model
        try:
            #该处代码参照ObtainAuthToken
            serializer = self.serializer_class(data=request.data,
                                               context={'request': request})
            serializer.is_valid(raise_exception=False)
            data = serializer.data

            username = data.get('username','')
            password = data.get('password','')
            if username == '':
                return ApiResponse(ApiState.failed, "请输入用户名", {})
            if password == '':
                return ApiResponse(ApiState.failed, "请输入有效的密码", {})

            us = model.objects.filter(username=data['username'])
            if us.count() > 0:
                return ApiResponse(ApiState.failed, '用户名已被注册', {})
            newObj = model.objects.create(username=username)
            newObj.set_password(password)
            newObj.save()
            token, created = self.Token.objects.get_or_create(user=newObj)
            data = self.registResponse(newObj, token.key)
            return ApiResponse(ApiState.success, "注册成功", data)
        except Exception as e:
            raise APIException(e)


    @action(methods=['post','options','get'], detail=False)
    @param_crawl(checker={
        'username': {'n': True},  # login | register
        'password': {'n': True},
    })
    def login(self, request, *args, **kwargs):
        # 该处代码参照ObtainAuthToken
        # import time
        # time.sleep(1)
        params = request.api_params

        user = self.authenticate(request,username=params.get('username',''),password=params.get('password',''))
        if user is None:
            return ApiResp(ApiState.failed, "用户名或密码错误", {})
        else:
            # token = self.Token.objects.get(user=user)
            try:
                token, created = self.Token.objects.get_or_create(user=user)
                data = self.loginResponse(user, token.key)
                return ApiResp(ApiState.success, "登录成功", data)
            except Exception as e:
                return ApiResp(ApiState.exception,'登录失败',data)



def AuthView(token):
    class FastAuthView(ApiAuthView):
        Token = token
    return FastAuthView

#
#
# class UserAuthView(viewsets.ViewSetMixin,ObtainAuthToken):
#     authModel = None
#
#
#     def loginResponse(self,user,token):
#         data = {'id':user.id}
#         data['token'] = token
#         return data
#
#     def registResponse(self,user,token):
#         return {'id': user.id, 'token': token}
#
#     @action(methods=['post','options','get'], detail=False)
#     def register(self, request):
#
#         if self.authModel is None:
#             return ApiResponse(ApiState.exception, "login error: TokenAuthView can not be None (authModel inherit from User model class )", "")
#         model = self.authModel
#         try:
#             #该处代码参照ObtainAuthToken
#             serializer = self.serializer_class(data=request.data,
#                                                context={'request': request})
#             serializer.is_valid(raise_exception=False)
#             data = serializer.data
#
#             username = data.get('username','')
#             password = data.get('password','')
#             if username == '':
#                 return ApiResponse(ApiState.failed, "请输入用户名", {})
#             if password == '':
#                 return ApiResponse(ApiState.failed, "请输入有效的密码", {})
#
#
#             us = model.objects.filter(username=data['username'])
#             if us.count() > 0:
#                 return ApiResponse(ApiState.failed, '用户名已被注册', {})
#
#             newObj = model.objects.create(username=username)
#             newObj.set_password(password)
#             newObj.save()
#             token, created = Token.objects.get_or_create(user=newObj)
#
#             data = self.registResponse(newObj, token.key)
#
#             return ApiResponse(ApiState.success, "注册成功", data)
#         except Exception as e:
#             raise APIException(e)
#
#
#     @action(methods=['post','options','get'], detail=False)
#     def login(self, request, *args, **kwargs):
#         # 该处代码参照ObtainAuthToken
#         # import time
#         # time.sleep(1)
#
#         serializer = self.serializer_class(data=request.data,
#                                            context={'request': request})
#         try:
#             serializer.is_valid(raise_exception=True)
#         except Exception as e:
#             return ApiResponse(ApiState.failed, "用户名或密码错误", {})
#
#         if self.authModel is None:
#             return ApiResponse(ApiState.exception, "login error: TokenAuthView can not be None (authModel inherit from User model class )", "")
#         try:
#             baseUser = serializer.validated_data['user']
#             token, created = Token.objects.get_or_create(user=baseUser)
#             data = self.loginResponse(baseUser,token.key)
#
#             return ApiResponse(ApiState.success, "登录成功", data)
#         except Exception as e:
#             return ApiResponse(ApiState.failed, "用户名或密码错误", {})
#
#



from rest_framework import authentication
from rest_framework import exceptions
from datetime import datetime,timedelta
import pytz

from rest_framework.exceptions import APIException


from rest_framework import exceptions
# from rest_framework.authentication import TokenAuthentication
from rest_framework import HTTP_HEADER_ENCODING, exceptions
# from rest_framework.authtoken.models import Token
from django.utils.translation import ugettext_lazy as _



class TokenAuthentication(authentication.BaseAuthentication):

    model = None
    """
    A custom token model may be used, but must have the following properties.

    * key -- The string identifying the token
    * user -- The user to which the token belongs
    """

    def authenticate(self, request):

        auth = get_authorization_header(request).split()
        if auth:
            token = auth[0]
            return self.authenticate_credentials(token)
        raise APIException(detail='loss token')



    def authenticate_credentials(self, key):
        try:
            key = str(key, 'utf-8')
            token = self.model.objects.select_related('user').get(key=key)
        except self.model.DoesNotExist:
            raise exceptions.AuthenticationFailed(_('Invalid token.'))

        if not token.user.is_active:
            raise exceptions.AuthenticationFailed(_('User inactive or deleted.'))

        return (token.user, token)

    def authenticate_header(self, request):
        return 'Token'

class ExpiredTokenAuthentication(authentication.BaseAuthentication):

    model = None
    """
    A custom token model may be used, but must have the following properties.

    * key -- The string identifying the token
    * user -- The user to which the token belongs
    """

    def authenticate(self, request):

        auth = get_authorization_header(request).split()
        if auth:
            token = auth[0]
            return self.authenticate_credentials(token)
        raise APIException(detail='loss token')



    def authenticate_credentials(self, key):
        try:
            key = str(key, 'utf-8')
            print("====dd",key)
            token = self.model.objects.select_related('user').get(key=key)
        except self.model.DoesNotExist:
            raise exceptions.AuthenticationFailed(_('Invalid token.'))

        if not token.user.is_active:
            raise exceptions.AuthenticationFailed(_('User inactive or deleted.'))

        # if self.userModel is not None:
        #     ecs = ['token','user',self.userModel]
        #     u = eval('.'.join(ecs))
        #     return (u,token)

        return (token.user, token)

    def authenticate_header(self, request):
        return 'Token'







#
#
# class SellerTokenAuthentication(authentication.BaseAuthentication):
#     def authenticate(self, request):
#         method = request.method
#         token = ''
#         # print request.POST
#         # print request.GET
#         # print request.data
#
#         if method == 'POST':
#
#             try:
#                 token = request.POST['token']
#             except:
#                 raise exceptions.AuthenticationFailed('need token')
#
#         if method == 'GET':
#             try:
#                 token = request.GET['token']
#             except:
#                 raise exceptions.AuthenticationFailed('need token')
#
#
#         if not token:
#             return None
#         try:
#             tokenObj = authentication.Token.objects.get(key=token)
#             user = tokenObj.user
#         except authentication.Token.DoesNotExist:
#             print('have not '+token)
#             raise exceptions.AuthenticationFailed('No such user')
#         return (user, None)




def get_authorization_header(request):
    """
    Return request's 'Authorization:' header, as a bytestring.

    Hide some test client ickyness where the header can be unicode.
    """
    auth = request.META.get('HTTP_TOKEN', b'')

    if isinstance(auth, type('')):
        # Work around django test client oddness
        auth = auth.encode(HTTP_HEADER_ENCODING)
    return auth


# Create your models here.
from django.db import models
from django.contrib import admin

from meapp.core.models import BaseUser, Token

# try:
#     from eeapp.base.models import BaseUser,Token
# except:
#     from base.models import BaseUser,Token


# from eeapp.base.models import BaseUser,Token


class Admin(BaseUser):
    mobile = models.CharField(max_length=20,verbose_name='手机号')

class AdminToken(Token):
    model = Admin
    user = models.OneToOneField(model, related_name='auth_token', on_delete=models.CASCADE, verbose_name="User",blank=True,null=True)
    pass

class Province(models.Model):
    province_id = models.CharField(
        max_length=6, default=None, verbose_name='省份编号')
    province = models.CharField(max_length=40, default=None, verbose_name='省份')
    order = models.IntegerField(verbose_name="排序", default=0)

    def __str__(self):
        return self.province


class City(models.Model):
    city_id = models.CharField(max_length=6, default=None, verbose_name='城市编号')
    city = models.CharField(max_length=50, default=None, verbose_name='市/县')
    father = models.ForeignKey(
        Province, on_delete=models.DO_NOTHING, related_name='cities', default=None, verbose_name='所属省份')
    order = models.IntegerField(verbose_name="排序", default=0)

    def __str__(self):
        return str(self.father) + ' ' +self.city


class Area(models.Model):
    area_id = models.CharField(
        max_length=50, default=None, verbose_name='地区编号')
    area = models.CharField(max_length=60, default=None, verbose_name='区')
    father = models.ForeignKey(
        City, on_delete=models.DO_NOTHING, related_name='areas', default=None, verbose_name='所属城市')
    order = models.IntegerField(verbose_name="排序", default=0)

    def __str__(self):
        return str(self.father) + ' ' +self.area


class Address(models.Model):
    order = models.IntegerField(verbose_name="排序", default=0)
    area = models.ForeignKey(Area, on_delete=models.DO_NOTHING)
    street = models.CharField(max_length=300, default='')
    address = models.CharField(max_length=300, default='')
    post_code = models.CharField(max_length=10, default='')


class CityInline(admin.TabularInline):
    model = City
    list = ['city', 'city_id']


class AreaInline(admin.TabularInline):
    model = Area
    list = ['area', 'area_id']


class ProvinceAdmin(admin.ModelAdmin):
    # fieldsets = [
    #     (None,               {'widget': ['question_text']}),
    #     ('Date information', {'widget': ['pub_date'], 'classes': ['collapse']}),
    # ]
    inlines = [CityInline]
    list_display = ('province', 'province_id')
    # list_filter = ['pub_date']
    # search_fields =


class CityAdmin(admin.ModelAdmin):
    inlines = [AreaInline]
    list_display = ('city', 'city_id')


class AreaAdmin(admin.ModelAdmin):
    list_display = ('area', 'area_id')


admin.site.register(Province, ProvinceAdmin)
admin.site.register(City, CityAdmin)
admin.site.register(Area, AreaAdmin)



# coding=utf-8
__author__ = 'zhuxietong'
import json
from rest_framework.exceptions import APIException
import re
from rest_framework.decorators import list_route
from django.core.files.base import ContentFile
import datetime
import functools

import random, string


def GenString(length=10):
    numOfNum = random.randint(1, length - 1)
    numOfLetter = length - numOfNum
    # 选中numOfNum个数字
    slcNum = [random.choice(string.digits) for i in range(numOfNum)]
    # 选中numOfLetter个字母
    slcLetter = [random.choice(string.ascii_letters) for i in range(numOfLetter)]
    # 打乱这个组合
    slcChar = slcNum + slcLetter
    random.shuffle(slcChar)
    # 生成密码
    genPwd = ''.join([i for i in slcChar])
    return genPwd


# n 是否必须参数 ， t 为参数转换类型  id 为正则检查id ,k 为返回的dict的key，model对象的属性对应

seller_login = {
    'username': {'n': True, 't': 'str', 'id': 'username'},
    'password': {'n': True, 't': 'str'},
    'email': {'n': True, 't': 'str'},
}

api_shop_create = {
    'tag': {},
    'area': {'k': 'area_id'},
    'address': {},
    'street': {'n': False},
}

PARAM_CHECK = {
    'default': {'reg': r'', 'lost_info': '缺少参数', 'dis_match_info': "参数检查错误", 'type': str},
    'username': {'reg': r'', 'lost_info': '缺少用户名', 'dis_match_info': "用户名格式不正确！", 'type': str},
    'password': {'reg': r'', 'lost_info': '缺少密码', 'dis_match_info': "密码包含特殊字符！", 'type': str},
    'address': {'reg': r'', 'lost_info': '缺少详细地址', 'dis_match_info': "地址内容检查无法通过", 'type': str},
    'street': {'reg': r'', 'lost_info': '缺少街道地址', 'dis_match_info': "街道内容检查错误！", 'type': str},
    'email': {'reg': r'', 'lost_info': '缺少邮箱', 'dis_match_info': "邮箱格式错误", 'type': str},
    'token': {'reg': r'', 'lost_info': '缺少token', 'dis_match_info': "token 错误！", 'type': str},
    'tag': {'reg': r'', 'lost_info': '缺少标签', 'dis_match_info': "标签内容验证失败", 'type': str},
    'name': {'reg': r'', 'lost_info': '缺少名称', 'dis_match_info': "名称出错", 'type': str},
    'mobile': {'reg': r'', 'lost_info': '缺少电话号码', 'dis_match_info': "电话号码错误", 'type': str},
    'price': {'reg': r'', 'lost_info': '缺少价格参数', 'dis_match_info': "价格格式错误", 'type': float},
    'description': {'reg': r'', 'lost_info': '缺少描述参数', 'dis_match_info': "描述验证失败", 'type': str},

}


def __get_nil_info(check_id):
    item = PARAM_CHECK[check_id]
    return item['lost_info']


def __get_dismatch_info(check_id):
    item = PARAM_CHECK[check_id]
    return item['dis_match_info']


def __get_checkValue(check_id, value):
    item = PARAM_CHECK[check_id]
    reg = item['reg']
    if reg:
        matchObj = re.match(reg, value)
        if matchObj:
            return value
        else:
            raise APIException(__get_dismatch_info(check_id))

    return value


def __convert(raw, type):
    value = raw
    if type == int:
        # print(int(raw))
        value = int(raw)
    if type == float:
        value = float(raw)
    if type == dict:
        try:
            obj = json.loads(raw)
            if isinstance(obj, dict):
                value = obj
            else:
                raise APIException('invalid dict field')
        except Exception as e:
            raise APIException(e)
    if type == list:
        try:
            obj = json.loads(raw)
            if isinstance(obj, list):
                value = obj
            else:
                raise APIException('invalid list field')
        except Exception as e:
            raise APIException(e)
    return value


def param_parser(request, checker):
    if not checker:
        return

    return_data = {}
    raw_data = request.GET
    if not raw_data:
        raw_data = request.POST
    if not raw_data:
        raw_data = request.data
    # print("========LLL",checker.items())
    for key, kv in checker.items():
        if not isinstance(kv, dict):
            raise APIException(
                '[server error]  checker has invalid value for key named %s , %s need a dict value' % (key, key))
        check_id = key
        need = True
        p_type = 'str'
        p_key = key

        if 'id' in kv:
            check_id = kv['id']
        if not check_id in PARAM_CHECK:
            check_id = 'default'
        if 'n' in kv:
            need = kv['n']
        if 't' in kv:
            p_type = kv['t']
        if 'k' in kv:
            p_key = kv['k']
        if need:
            # print('10')

            if not key in raw_data:
                # print('11')
                raise APIException(__get_nil_info(check_id) + "  '" + key + "'")
            else:

                # print('12')
                value = raw_data[key]

                if value:
                    checkedV = __get_checkValue(check_id, value)
                    # print("000000aaakkkkkkkkd", value,raw_data,key)
                    return_data[p_key] = __convert(checkedV, p_type)
                    # print('13')
                else:
                    # print('14')
                    raise APIException(__get_nil_info(check_id))
        else:
            if key in raw_data:
                # print('15')
                value = raw_data[key]
                if value:
                    # print('16')
                    checkedV = __get_checkValue(check_id, value)
                    return_data[p_key] = __convert(checkedV, p_type)
                else:
                    # print('17')
                    raise APIException(__get_nil_info(check_id))

        request.api_params = return_data
    # return (True,'ok',return_data)


def param_crawl(checker=None, keys=None, do_route=False):
    try:
        if not checker:
            keys = ['get'] if (keys is None) else keys
            if keys:
                checker = {}
                for key in keys:
                    checker[key] = {}
        if do_route:
            def decrector(f):
                @functools.wraps(f)
                def fct(self, *args, **kwargs):
                    param_parser(self.request, checker=checker)
                    return f(self, *args, **kwargs)

                return fct

            return decrector
        else:
            def decrector(f):
                @functools.wraps(f)
                def fct(self, *args, **kwargs):
                    param_parser(self.request, checker=checker)
                    try:
                        return f(self, *args, **kwargs)
                    except Exception as e:
                        raise Exception(e)

                return fct

            return decrector
    except Exception as e:
        raise Exception(e)


def __image_load(request, IClass, fileKey, param):
    if request.method == 'POST':
        param_parser(request, param)
        object = IClass()
        save(object, request.api_params)

        try:
            file_content = ContentFile(request.FILES[fileKey].read())
            str = request.FILES[fileKey].name
            date = datetime.datetime.now()
            timeStr = date.strftime('%y%m%d%H%M%S')
            name = timeStr + GenString(length=15)
            filename = name + '.' + str.split('.')[-1]
            object.slug = name
            object.image.save(filename, file_content)
            request.param_image = object
        except Exception as e:
            raise APIException(e)


def load_image(IClass, fileKey, param):
    def decrector(f):
        @functools.wraps(f)
        def fct(self, *args, **kwargs):
            __image_load(self.request, IClass, fileKey, param)
            return f(self, *args, **kwargs)

        return fct

    return decrector


# def param_crawl(checker=None,keys=None,do_route=True):
#     if not checker:
#         keys = ['get'] if (keys is None) else keys
#         if keys:
#             checker = {}
#             for key in keys:
#                 checker[key] = {}
#     if do_route:
#             @list_route(methods=['get','post',])
#             def decrector(f):
#                 def fct(self,request, *args, **kwargs):
#                     print request
#                     param_parser(request,checker=checker)
#                     return f(self,request,*args, **kwargs)
#                 return fct
#             return decrector
#     else:
#             def decrector(f):
#                 def fct(self,request, *args, **kwargs):
#                     print request
#                     param_parser(request,checker=checker)
#                     return f(self,request,*args, **kwargs)
#                 return fct
#             return decrector

def save(obj, atrributes):
    for key, value in atrributes.items():
        if hasattr(obj, key):
            setattr(obj, key, value)



#coding=utf-8
__author__ = 'zhuxietong'

from django.contrib.auth import authenticate
from rest_framework import serializers
# from rest_framework_expiring_authtoken.models import ExpiringToken


from django.utils.translation import ugettext_lazy as _

from rest_framework.exceptions import APIException


from meapp.location.models import *

class AdminSerializer(serializers.ModelSerializer):
    # avatar = serializers.SerializerMethodField()
    # def get_avatar(self,instance):
    #     try:
    #         return instance.avatar.url
    #     except Exception as e:
    #         return ''

    class Meta:
        model = Admin
        fields = ('username','id')

class UserLoginSerializer(serializers.Serializer):
    username = serializers.CharField()
    password = serializers.CharField(style={'input_type': 'password'})

    def validate(self, attrs):
        username = attrs.get('username')
        password = attrs.get('password')

        if username and password:
            user = authenticate(username=username, password=password)

            if user:
                if not user.is_active:
                    msg = _('User base is disabled.')
                    raise APIException('User base is disabled.')
            else:
                msg = _('Unable to log in with provided credentials.')
                raise APIException('Unable to log in with provided credentials.')
        else:
            msg = _('Must include "username" and "password".')
            raise APIException('Must include "username" and "password".')

        attrs['user'] = user.account.seller
        return attrs


# class GoodsImageSerializer(serializers.ModelSerializer):
#     class Meta:
#         model = GoodsImage
#         fields = ('image','thumbnail','type')

# class GoodsBookSerializer(serializers.ModelSerializer):
#     # fine-uploader = serializers.SerializerMethodField()
#     #
#     # def get_images(self,instance):
#     #     return GoodsImageSerializer(many=True,data=instance.fine-uploader).data
#     fine-uploader = GoodsImageSerializer(many=True)
#     class Meta:
#         model = Goods
#         depth = 1
#         fields = ('name','description','price','fine-uploader')

class ProvinceSerializer(serializers.ModelSerializer):
    class Meta:
        model = Province
        depth = 1
        fields = ('province','province_id',)

class CitySerializer(serializers.ModelSerializer):
    province = serializers.SerializerMethodField()
    province_id = serializers.SerializerMethodField()

    class Meta:
        model = City
        fields = ('city','city_id','province','province_id',)
    def get_province(self,instance):
        return instance.father.province

    def get_province_id(self,instance):
        return instance.father.province_id

class AreaSerializer(serializers.ModelSerializer):
    city = serializers.SerializerMethodField()
    city_id = serializers.SerializerMethodField()
    class Meta:
        model = Area
        fields = ('area','area_id','city','city_id',)

    def get_city(self,instance):
        return instance.father.city

    def get_city_id(self,instance):
        return instance.father.city_id




from django.db import models
import json

from django.utils.encoding import (
    force_bytes, force_text, python_2_unicode_compatible, smart_text,
)
from django.utils import six
from django.conf import settings
from ..model.Images import ImageModel

import ast


class ImageSetField(models.TextField):
    # __metaclass__ = models.SubfieldBase
    description = "Stores a python list"

    def __init__(self, *args, **kwargs):
        super(ImageSetField, self).__init__(*args, **kwargs)

    @property
    def urls(self):
        return ['name',"page"]

    def from_db_value(self, value, expression, connection, context):
        return self.to_python(value)

    def to_python(self, value):
        if not value:
            value = []

        if isinstance(value, list):
            return value

        return ast.literal_eval(value)

    def get_prep_value(self, value):
        if isinstance(value,str):
            print('str --- ')

        if isinstance(value,list):
            pass

        if isinstance(value,object):
            print('obj --- ')

        if value is None:
            return value

        return str(value)  # use str(value) in Python 3




from rest_framework.exceptions import APIException

from rest_framework import viewsets
from rest_framework.viewsets import ViewSet
from rest_framework.decorators import action
from .auth import AdminTokenAuth


from meapp.location import models
from meapp.api.response import ApiResp,ApiState
from meapp.api import response
from meapp.api.view.authView import ApiAuthView

#

from . import serializer


class AdminAuthView(ApiAuthView):
    Token = models.AdminToken
    def loginResponse(self,user,token):
        data = serializer.AdminSerializer(user.manager).data
        data['token'] = token
        # ManagerSerializer(user.manager)
        return data

    def registResponse(self,user,token):
        return {'userid': user.id, 'token': token}

class AdminView(ViewSet):

    @action(methods=['post','get','options'], detail=False,authentication_classes=[AdminTokenAuth])
    def test(self, request):
        print(request.user)
        return ApiResp(ApiState.failed, "上传失败", '')




class ProvinceListView(viewsets.ModelViewSet):
    queryset = models.Province.objects.all()
    serializer_class = serializer.ProvinceSerializer
    pagination_class = response.OnePagination


class CityListView(viewsets.ModelViewSet):
    serializer_class = serializer.CitySerializer
    pagination_class = response.OnePagination

    def get_queryset(self):
        try:
            province_id = self.request.GET['province_id']
        except:
            raise APIException('lost province_id')
        try:
            province = models.Province.objects.get(pk=province_id)
        except:
            raise APIException('error province_id')
        # cities = province.cities
        # print(cities)
        return province.cities.all()


class AreaListView(viewsets.ModelViewSet):
    serializer_class = serializer.AreaSerializer
    pagination_class = response.OnePagination
    def get_queryset(self):
        try:
            city_id = self.request.GET['city_id']
        except:
            raise APIException('lost city_id')
        try:
            city = models.City.objects.get(pk=city_id)
        except:
            raise APIException('error city_id')
        return city.areas.all()


from rest_framework.exceptions import APIException
from rest_framework import viewsets,generics


from rest_framework.decorators import list_route

from ..response import ApiResponse,ApiState
from ..crawl import param_crawl,save



from django.http import Http404
from django.shortcuts import get_object_or_404 as _get_object_or_404


from rest_framework.response import Response

def get_object_or_404(queryset, *filter_args, **filter_kwargs):
    """
    Same as Django's standard shortcut, but make sure to also raise 404
    if the filter_kwargs don't match the required types.
    """
    try:
        return _get_object_or_404(queryset, *filter_args, **filter_kwargs)
    except (TypeError, ValueError):
        raise Http404


class ModelView(viewsets.ModelViewSet):
    @list_route(methods=['post','get'])
    @param_crawl(checker={
        'id':{'n':True},
    })
    def delete(self,request):
        print(request.api_params['id'])
        try:
            self.serializer_class.Meta.model.objects.get(pk=request.api_params['id'])
        except Exception as e:
            raise APIException(e)
        return ApiResponse(detail='delete ok')

    @list_route(methods=['post','get'])
    def new(self, request, *args, **kwargs):
        return super(ModelView,self).create(request,*args, **kwargs)

    @list_route(methods=['post','get'])
    def update(self, request, *args, **kwargs):
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        return Response(serializer.data)


    def get_object(self):

        queryset = self.filter_queryset(self.get_queryset())

        filter_kwargs = {'pk':self.request.data['id']}
        obj = get_object_or_404(queryset, **filter_kwargs)

        self.check_object_permissions(self.request, obj)

        return obj



from rest_framework import generics, mixins, views


class ModelEditView(mixins.CreateModelMixin,
                   mixins.RetrieveModelMixin,
                   mixins.UpdateModelMixin,
                   mixins.DestroyModelMixin,
                   viewsets.GenericViewSet):

    @list_route(methods=['post','get'])
    @param_crawl(checker={
        'id':{'n':True},
    })
    def delete(self,request):
        print(request.api_params['id'])
        try:
            self.serializer_class.Meta.model.objects.get(pk=request.api_params['id'])
        except Exception as e:
            raise APIException(e)
        return ApiResponse(detail='delete ok')

    @list_route(methods=['post','get'])
    def new(self, request, *args, **kwargs):
        return super(ModelEditView,self).create(request,*args, **kwargs)

    @list_route(methods=['post','get'])
    def change(self, request, *args, **kwargs):
        return super(ModelEditView,self).update(request,*args, **kwargs)

    @list_route(methods=['post','get'])
    def update(self, request, *args, **kwargs):
        partial = kwargs.pop('partial', False)
        instance = self.get_object()
        serializer = self.get_serializer(instance, data=request.data, partial=partial)
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)
        return Response(serializer.data)


    def get_object(self):

        queryset = self.filter_queryset(self.get_queryset())

        filter_kwargs = {'pk':self.request.data['id']}
        obj = get_object_or_404(queryset, **filter_kwargs)

        self.check_object_permissions(self.request, obj)

        return obj


from rest_framework.routers import DefaultRouter,SimpleRouter
from django.conf.urls import url,include

from .views import *

# router1 = SimpleRouter(trailing_slash=False)

router = DefaultRouter(trailing_slash=False)
router.register('province',ProvinceListView)
router.register('cities',CityListView,base_name='cities')
router.register('areas',AreaListView,base_name='areas')
router.register('admintest',AdminAuthView,base_name='admintest')
router.register('admin',AdminView,base_name='admindemo')

urlpatterns = [
    url(r'^',include(router.urls)),
]



from django.conf import settings

DATABASE_MAPPING = settings.DATABASE_APPS_MAPPING


class DatabaseAppsRouter(object):
    """
    A router to control all database operations on models for different
    databases.

    In case an app is not set in settings.DATABASE_APPS_MAPPING, the router
    will fallback to the `default` database.

    Settings example:

    DATABASE_APPS_MAPPING = {'app1': 'db1', 'app2': 'db2'}
    """

    def db_for_read(self, model, **hints):
        """"Point all read operations to the specific database."""
        if model._meta.app_label in DATABASE_MAPPING:
            return DATABASE_MAPPING[model._meta.app_label]
        return None

    def db_for_write(self, model, **hints):
        """Point all write operations to the specific database."""
        if model._meta.app_label in DATABASE_MAPPING:
            return DATABASE_MAPPING[model._meta.app_label]
        return None

    def allow_relation(self, obj1, obj2, **hints):
        """Allow any relation between apps that use the same database."""
        db_obj1 = DATABASE_MAPPING.get(obj1._meta.app_label)
        db_obj2 = DATABASE_MAPPING.get(obj2._meta.app_label)
        if db_obj1 and db_obj2:
            if db_obj1 == db_obj2:
                return True
            else:
                return False
        return None

    def allow_syncdb(self, db, model):
        """Make sure that apps only appear in the related database."""

        if db in DATABASE_MAPPING.values():
            return DATABASE_MAPPING.get(model._meta.app_label) == db
        elif model._meta.app_label in DATABASE_MAPPING:
            return False
        return None

    def allow_migrate(self, db, app_label, model=None, **hints):
        """
        Make sure the auth app only appears in the 'auth_db'
        database.
        """
        if db in DATABASE_MAPPING.values():
            return DATABASE_MAPPING.get(app_label) == db
        elif app_label in DATABASE_MAPPING:
            return False
        return None


#!/usr/bin/env python
# -*- coding:utf-8 -*-
import random, string


def GenString(length=10):
    numOfNum = random.randint(1, length - 1)
    numOfLetter = length - numOfNum
    # 选中numOfNum个数字
    slcNum = [random.choice(string.digits) for i in range(numOfNum)]
    # 选中numOfLetter个字母
    slcLetter = [random.choice(string.ascii_letters) for i in range(numOfLetter)]
    # 打乱这个组合
    slcChar = slcNum + slcLetter
    random.shuffle(slcChar)
    # 生成密码
    genPwd = ''.join([i for i in slcChar])
    return genPwd


def GenDsigitalCode(length=4):
    code = ''
    limit = length + 1
    for num in range(1, limit):
        code = code + str(random.randint(0, 9));
    return code





def crossappend(response):
    response["Access-Control-Allow-Origin"] = "*"
    response["Access-Control-Allow-Methods"] = "GET,POST,PUT,DELETE,OPTIONS"
    response["Access-Control-Max-Age"] = "1000"
    response[
        "Access-Control-Allow-Headers"] = "Accept,Accept-Language,Content-Language,Content-Type,X-Custom-Header,x-requested-with,origin,authorization,x-csrftoken,token"


def crossdomain(prefix):
    def cross_decorator(f):
        def wrapper(*args, **kw):
            response = f(*args, **kw)
            crossappend(response)
            return response
        return wrapper
    return cross_decorator



__author__ = 'zhuxietong'
from rest_framework import pagination
from rest_framework.response import Response
from rest_framework.renderers import JSONRenderer


from enum import Enum, unique
import json
from .crossdomain import crossdomain,crossappend

from django.conf import settings

ApiCode = 'status'
ApiData = 'data'
ApiMessage = 'message'

ApiPageSizeKey = 'pagesize'
ApiPageKey = 'page'

try:
    if settings.ApiCode is not None:
        ApiCode = settings.ApiCode
except Exception as e:
    pass
try:
    if settings.ApiData is not None:
        ApiData = settings.ApiData
except Exception as e:
    pass

try:
    if settings.ApiMessage is not None:
        ApiMessage = settings.ApiMessage
except Exception as e:
    pass

try:
    if settings.ApiPageSizeKey is not None:
        ApiPageSizeKey = settings.ApiPageSizeKey
except Exception as e:
    pass

try:
    if settings.ApiPageKey is not None:
        ApiPageKey = settings.ApiPageKey
except Exception as e:
    pass




class ApiState(Enum):
    failed = 0
    success = 1
    unknown = 2
    exception = 3
    invalid_wx_auth = 31

@crossdomain('')
def ApiResponse(state=ApiState.success, detail='', data=None):
    response = Response({ApiCode: state.value, ApiMessage: detail, ApiData: data})
    return response

@crossdomain('')
def ApiResp(state, detail, data):
    response = Response({ApiCode: state.value, ApiMessage: detail, ApiData: data})
    return response

from rest_framework.exceptions import APIException
from rest_framework.response import Response



from django.http import JsonResponse
from rest_framework import status

@crossdomain('')
def Api404(request, exception, *args, **kwargs):

    data = {
        'detail': 'Bad Request (404)',
        'status':ApiState.failed.value,
        'data':{}
    }
    response = JsonResponse(data, status=status.HTTP_404_NOT_FOUND)
    return response


def isRootData(data):
    validNum = 0
    if not ApiCode in data:
        return False
    if not ApiMessage in data:
        return False
    if not  ApiData in data:
        return False
    return  True


class ApiJSONRenderer(JSONRenderer):
    def render(self, data, accepted_media_type=None, renderer_context=None):
        # state = 200


        isRoot = isRootData(data)
        if not isRoot:
            data = {
                ApiCode:ApiState.success.value,
                ApiData:data,
                ApiMessage:'succss'
            }

        # if ApiCode in data:
        #     state = data[ApiCode]
        #     if state == ApiState.exception.value:
        #         err = True
        #     if state == ApiState.failed.value:
        #         err = True
        # else:
        #     data = {
        #         ApiMessage: 'success',
        #         ApiCode: ApiState.success.value,
        #         ApiData: data
        #     }
        #
        # if err:
        #
        #     detail = 'error'
        #     if isinstance(data,str):
        #         detail = data
        #     elif isinstance(data,dict):
        #         infos = []
        #         for key,value in data.items():
        #             if key != ApiCode:
        #                 infos.append(json.dumps(value))
        #
        #         detail = ';'.join(infos)
        #     else:
        #         detail = json.dumps(data)
        #     err_data = {ApiCode: 0}
        #     err_data[ApiMessage] = detail
        #     err_data[ApiData] = ''
        #     data = err_data
        #
        # else:
        #     detail = 'success'
        #
        #     obj = data
        #     if ApiData in data:
        #         obj = data[ApiData]
        #     if ApiMessage in data:
        #         detail = data[ApiMessage]
        #     data = {
        #         ApiData: obj,
        #         ApiCode: 1,
        #         ApiMessage: detail
        #     }
        # # print(renderer_context)
        # crossappend(renderer_context["response"])
        print("=======DDD---DD",data)
        return super(ApiJSONRenderer, self).render(data=data, accepted_media_type=accepted_media_type, renderer_context=renderer_context)


class ListPagination(pagination.PageNumberPagination):
    page_size = 10
    page_size_query_param = ApiPageSizeKey
    page_query_param = ApiPageKey

    max_page_size = 1000

    @crossdomain('')
    def get_paginated_response(self, data):

        content_data = {
            'list': data,
            'total': self.page.paginator.count,
            # 'links':{
            #    'next': self.get_next_link(),
            #    'previous': self.get_previous_link()
            # },
        }
        return  ApiResponse(state=ApiState.success,detail='success',data=content_data)


class TablePagination(pagination.PageNumberPagination):
    page_size = 10
    page_size_query_param = 'length'
    max_page_size = 1000
    page_query_param = 'start'

    @crossdomain('')
    def get_paginated_response(self, data):
        # print('page:%s'%self.page.paginator.num_pages)

        print(self.request.query_params)
        page_size = self.get_page_size(self.request)
        p_number = self.request.query_params.get(self.page_query_param, 1)
        page_number = int(p_number)+1

        draw = 0
        try:
            draw = self.request.GET['draw']
            draw = int(draw)
        except Exception as e:
            print(e)

        content_data = {
            'list': data,
            'total': self.page.paginator.count,
            'size': int(page_size),
            'start': page_number,
            'draw': draw
            # 'links':{
            #    'next': self.get_next_link(),
            #    'previous': self.get_previous_link()
            # },
        }

        # response = Response(content_data)
        # return response
        return  ApiResponse(state=ApiState.success,detail='success',data=content_data)



    def paginate_queryset(self, queryset, request, view=None):
        """
        Paginate a queryset if required, either returning a
        page object, or `None` if pagination is not configured for this view.
        """
        page_size = self.get_page_size(request)
        if not page_size:
            return None

        import six
        from django.core.paginator import InvalidPage
        from rest_framework.exceptions import NotFound
        paginator = self.django_paginator_class(queryset, page_size)
        p_number = request.query_params.get(self.page_query_param, 1)

        pg_number = int(p_number)/page_size
        # step = int(1)
        page_number = pg_number + 1
        if page_number in self.last_page_strings:
            page_number = paginator.num_pages

        try:
            self.page = paginator.page(page_number)
        except InvalidPage as exc:
            msg = self.invalid_page_message.format(
                page_number=page_number, message=six.text_type(exc)
            )
            raise NotFound(msg)

        if paginator.num_pages > 1 and self.template is not None:
            # The browsable API should display pagination controls.
            self.display_page_controls = True

        self.request = request
        return list(self.page)


class OnePagination(pagination.PageNumberPagination):
    page_size = 10000
    page_size_query_param = 'page_size'
    max_page_size = 10000

    def get_paginated_response(self, data):
        content_data = {
            'list': data,
            'total': self.page.paginator.count,
            # 'links':{
            #    'next': self.get_next_link(),
            #    'previous': self.get_previous_link()
            # },
        }
        return  ApiResponse(state=ApiState.success,detail='success',data=content_data)



from rest_framework import serializers

from ..models import TestModel,OptionsModel

class TestModelSerializer(serializers.ModelSerializer):
    images = serializers.SerializerMethodField()

    class Meta:
        model = TestModel
        fields = ('name','email','covers','sex','images','hobbies','time')

    def get_images(self,instance):
        return instance.covers


class OptionModelSerializer(serializers.ModelSerializer):
    class Meta:
        model = OptionsModel
        fields = ('name','value','id')



from rest_framework.viewsets import ViewSet,ModelViewSet
from rest_framework.decorators import action

from ..model.Images import ImageModel
from ..models import TestModel,OptionsModel

from .serializer import TestModelSerializer,OptionModelSerializer
from meapp.api.response import ApiState,ApiResponse,OnePagination



class ImageView(ViewSet):
    @action(methods=['post','options'], detail=False)
    def upload(self, request):
        try:
            img = ImageModel(image=request.FILES.get('file'))
            img.save()
            resp = ApiResponse(ApiState.success, "上传成功",img.url)
            return resp
        except Exception as e:
            return ApiResponse(ApiState.failed, "上传失败", '')






class OptionsView(ModelViewSet):
    serializer_class = OptionModelSerializer
    pagination_class = OnePagination
    def get_queryset(self):
        objs = OptionsModel.objects.all()
        return objs


class TestModelView(ModelViewSet):
    serializer_class = TestModelSerializer
    def get_queryset(self):
        objs = TestModel.objects.all()
        # try:
        #     order_index = self.request.GET['order[0][column]']
        #     order_p = "columns[%s][data]" % order_index
        #     order = self.request.GET[order_p]
        #
        #     order_up =  self.request.GET['order[0][dir]']
        #     if order_up == 'desc':
        #         objs = TestModel.objects.order_by("-%s"%order)
        #     else:
        #         objs = TestModel.objects.order_by(order)
        #
        #     key = self.request.GET['search[value]']
        #     print('order is %s'%order)
        #     print('search:%s' % key)
        #     objs = objs.filter(name__contains=key)
        # except Exception as e:
        #     print(e)
        #
        # objs = objs.filter(name__contains='')
        return objs





import json
from meapp.api.response import ApiState,ApiResp
from aliyunsdkcore.client import AcsClient
from aliyunsdkcore.request import CommonRequest

from django.conf import settings

AliYunConfig = {
    'AccessKey':'',
    'Secret':'',
    'protocolType':'https'
}


try:
    if settings.ALIYUN is not None:
        AliYunConfig = settings.ALIYUN
except Exception as e:
    pass


if not AliYunConfig['AccessKey']:
    raise Exception('please set AccessKey for ALIYUN at setting')

if not AliYunConfig['Secret']:
    raise Exception('please set Secret for ALIYUN at setting')

if not AliYunConfig['protocolType']:
    AliYunConfig['protocolType'] = "http"


class SingleSMS(object):
    mobiles = []
    signName = ''#老朱管理
    templateCode = ''#SMS_66460074
    templateParam = ''#

    protocolType = AliYunConfig['protocolType']

    def __init__(self,**params):
        for (key, value) in params.items():
            ext = 'self.%s = "%s"' % (key, value)
            # print(ext)
            exec(ext)

    def send(self,mobile,templateParam):
        code = 'except'

        try:
            client = AcsClient(AliYunConfig['AccessKey'], AliYunConfig['Secret'], 'default')

            request = CommonRequest()
            request.set_accept_format('json')
            request.set_domain('dysmsapi.aliyuncs.com')
            request.set_method('POST')

            request.set_protocol_type('https')  # https | http
            request.set_version('2017-05-25')
            request.set_action_name('SendSms')

            request.add_query_param('PhoneNumbers', mobile)
            request.add_query_param('SignName', self.signName)
            request.add_query_param('TemplateCode', self.templateCode)
            request.add_query_param('Format', 'json')

            if isinstance(templateParam,dict):
                #templateParam = {'code':'1234'}
                request.add_query_param('TemplateParam', json.dumps(templateParam))
            else:
                request.add_query_param('TemplateParam', templateParam)

            try:
                response = client.do_action_with_exception(request)
                rsp = str(response, encoding='utf-8')
                obj = json.loads(rsp);
                code = obj['Code'] + ""
                code = code.lower()
                return (ApiState.success,code)
            except Exception as e:
                return (ApiState.exception, code)

        except Exception as e:
            return (ApiState.exception,code)



def SingleSMSTemplate(SignName,TemplateCode):

    class Template(SingleSMS):
        signName = SignName
        templateCode = TemplateCode

    return Template


# Create your models here.
from django.contrib.auth.models import User, UserManager, AbstractBaseUser
from rest_framework.authtoken.models import Token

from django.utils import timezone
from django.contrib.auth.validators import UnicodeUsernameValidator

from .fields.ImageSetField import ImageSetField
from meapp.api.response import ApiResp,ApiState

import binascii
import os

from django.conf import settings
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import ugettext_lazy as _



SUPERCODE = 'isadmin'

try:
    if settings.SUPERCODE is not None:
        SUPERCODE = settings.SUPERCODE
except Exception as e:
    pass

from meapp.tools.gencode import GenDsigitalCode

# try:
#     from tools.gencode import GenDsigitalCode
# except Exception as e:
#     from ..tools.gencode import GenDsigitalCode

# print(__name__)


class UManager(UserManager):
    def get_queryset(self):
        return super().get_queryset().filter(VALID=True)

class SafeManager(models.Manager):
    def get_queryset(self):
        return super().get_queryset().filter(VALID=True)


class Model(models.Model):
    VALID = models.BooleanField(default=True)
    objects = SafeManager()
    class Meta:
        abstract = True

class CodeRecord(Model):
    mobile = models.CharField(max_length=11,verbose_name='手机号')
    code = models.CharField(max_length=6,verbose_name='验证码')
    tag = models.CharField(max_length=32,verbose_name='标签',default='unkown')
    created = models.DateTimeField(_("Created"), auto_now_add=True)
    def save(self, *args, **kwargs):
        if not self.code:
            self.code = GenDsigitalCode(4)
        super(CodeRecord, self).save(*args, **kwargs)

    @staticmethod
    def isvalid(mobile,code,tag):
        valid = False
        try:
            code = code + ''
            records = CodeRecord.objects.filter(code=code,mobile=mobile,tag=tag)
            if len(records) < 1:
                return(False, '验证码无效')
            else:
                record = records[0]

            time = record.created
            # from datetime import datetime, timedelta, tzinfo
            # datetime()
            now = timezone.now()
            distance = now - time
            seconds = distance.total_seconds()
            if seconds > 60:
                return (False,'验证码已失效')
            else:
                return (True,'验证成功')
        except CodeRecord.DoesNotExist:
            return (False, '验证码无效')

    @staticmethod
    def fastCheeck(request):
        try:
            params = request.api_params
            code = params.get('code','')
            mobile = params.get('mobile','')
            tag = params.get('type','')
            result = CodeRecord.isvalid(mobile,code,tag)
            if result[0]:
                return ApiResp(ApiState.success,result[1],'')
            else:
                return ApiResp(ApiState.failed,result[1],'')
        except Exception as e:
            return ApiResp(ApiState.exception, '验证异常', '')






class Account(User):
    display_name = models.CharField(max_length=255,default='',verbose_name='昵称')
    VALID = models.BooleanField(default=True)
    role = models.CharField(max_length=10,default='base')

    wxopenid = models.CharField(max_length=64,default='')
    objects = UManager()

    def get_role(self):
        return 'base'

    def auth_token(self):
        token,created = Token.objects.get_or_create(user=self)
        return  token.key

    class Meta(User.Meta):
        abstract = True
        permissions = (
            ('base_account_create', 'base create new base'),
            ('base_account_fix', 'base edit other base'),
            ('base_account_delete', 'base drop other base')
        )

    def save(self, *args, **kwargs):
        if not self.username:
            if self.email:
                self.username = self.email
        if not self.role:
            self.role = self.get_role()
        super(Account, self).save(*args, **kwargs)
        token = Token.objects.get_or_create(user=self)



#AbstractUser
class BaseUser(AbstractBaseUser):
    display_name = models.CharField(max_length=255,default='',verbose_name='昵称')
    VALID = models.BooleanField(default=True)

    role = models.CharField(max_length=10,default='base')
    wxopenid = models.CharField(max_length=64,default='')
    objects = UManager()
    # objects = UserManager()


    username_validator = UnicodeUsernameValidator()
    username = models.CharField(
        _('username'),
        max_length=150,
        unique=True,
        help_text=_('Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.'),
        validators=[username_validator],
        error_messages={
            'unique': _("A user with that username already exists."),
        },
    )
    first_name = models.CharField(_('first name'), max_length=30, blank=True)
    last_name = models.CharField(_('last name'), max_length=150, blank=True)
    email = models.EmailField(_('email address'), blank=True)
    is_staff = models.BooleanField(
        _('staff status'),
        default=False,
        help_text=_('Designates whether the user can log into this admin site.'),
    )
    is_active = models.BooleanField(
        _('active'),
        default=True,
        help_text=_(
            'Designates whether this user should be treated as active. '
            'Unselect this instead of deleting accounts.'
        ),
    )
    date_joined = models.DateTimeField(_('date joined'), default=timezone.now)


    EMAIL_FIELD = 'email'
    USERNAME_FIELD = 'username'
    REQUIRED_FIELDS = ['email']

    class Meta:
        verbose_name = _('user')
        verbose_name_plural = _('users')
        abstract = True


    def get_role(self):
        return 'base'

    def auth_token(self):
        token,created = Token.objects.get_or_create(user=self)
        return  token.key

    # class Meta(User.Meta):
    #     abstract = True
    #     # permissions = (
    #     #     ('base_account_create', 'base create new base'),
    #     #     ('base_account_fix', 'base edit other base'),
    #     #     ('base_account_delete', 'base drop other base')
    #     # )

    def save(self, *args, **kwargs):
        if not self.username:
            if self.email:
                self.username = self.email
        if not self.role:
            self.role = self.get_role()
        super(BaseUser, self).save(*args, **kwargs)
        # token = Token.objects.get_or_create(user=self)



class WXUserAuth(models.Model):
    openid = models.CharField(max_length=256,verbose_name='微信openid')
    code = models.CharField(max_length=256,blank=True,verbose_name='code')
    access_token = models.CharField(max_length=256,blank=True,verbose_name='access_token')
    refresh_token = models.CharField(max_length=256,blank=True,verbose_name='refresh_token')
    create_time = models.DateTimeField(auto_now_add=True,verbose_name='创建时间')
    update_time = models.DateTimeField(auto_now=True,verbose_name='更新时间')
    VALID = models.BooleanField(default=True)


class WXAuthLog(models.Model):
    user = models.ForeignKey(WXUserAuth,on_delete=models.CASCADE,verbose_name='用户')
    openid = models.CharField(max_length=256,verbose_name='微信openid')
    access_token = models.CharField(max_length=256,verbose_name='access_token')
    refresh_token = models.CharField(max_length=256,verbose_name='refresh_token')
    create_time = models.DateTimeField(auto_now_add=True,verbose_name='创建时间')
    VALID = models.BooleanField(default=True)






class Log(models.Model):

    content = models.CharField(max_length=255,verbose_name='日志内容',blank=True)
    time = models.DateTimeField(verbose_name='时间',auto_now_add=True)
    mark = models.CharField(max_length=255,verbose_name='操作者',blank=True)

    class Meta:
        abstract = False





class OptionsModel(models.Model):
    name = models.CharField(max_length=64,blank=False,verbose_name='名称')
    value = models.CharField(max_length=20,blank=False,verbose_name='值')



class TestModel(models.Model):
    hobby_choice = (
        ('music', '音乐'),
        ('sport', '运动'),
        ('film', '电影'),
        ('tour', '旅游'),
    )


    email = models.EmailField(max_length=70,blank=True,verbose_name='邮箱')
    name = models.CharField(max_length=20,blank=False,verbose_name='名称')
    covers = ImageSetField(blank=True,verbose_name='封面')
    sex = models.CharField(choices=(('boy','男'),('girl','女'),('unknown','未知')),verbose_name='性别',max_length=10,default='unknown')
    hobbies = models.CharField(verbose_name='爱好',max_length=50,blank=True)
    time = models.TimeField(verbose_name='时间')
    birthday = models.DateField(verbose_name='生日')


    over_time = models.DateTimeField(blank=True,verbose_name='过期时间')




@python_2_unicode_compatible
class Token(models.Model):

    @classmethod
    def getModel(cls):
        return cls.model
    #
    # @property
    # def user(self):
    #     print("___++D++D+D+",self.model)
    #     return models.OneToOneField(self.model, related_name='auth_token',on_delete=models.CASCADE, verbose_name=_("User"),blank=True,null=True)
    #
    # @user.setter
    # def user(self):
    #     print("___++D++D+D+", self.model)
    #     return models.OneToOneField(self.model, related_name='auth_token', on_delete=models.CASCADE,
    #                                 verbose_name=_("User"), blank=True, null=True)

    """
    The default authorization token model.
    """
    key = models.CharField(_("Key"), max_length=40, primary_key=True)

    model = TestModel
    user = models.OneToOneField(model, related_name='auth_token', on_delete=models.CASCADE, verbose_name=_("User"),blank=True,null=True)
    created = models.DateTimeField(_("Created"), auto_now_add=True)
    update_time = models.DateTimeField(verbose_name='更新时间',auto_now=True)

    class Meta:
        # Work around for a bug in Django:
        # https://code.djangoproject.com/ticket/19422
        #
        # Also see corresponding ticket:
        # https://github.com/encode/django-rest-framework/issues/705
        # abstract =
        abstract = True
        verbose_name = _("Token")
        verbose_name_plural = _("Tokens")

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = self.generate_key()
        return super(Token, self).save(*args, **kwargs)

    def generate_key(self):
        return binascii.hexlify(os.urandom(20)).decode()

    def __str__(self):
        return self.key




from rest_framework.routers import DefaultRouter,SimpleRouter
from django.conf.urls import url,include

from .views import *

# router1 = SimpleRouter(trailing_slash=False)

router = DefaultRouter(trailing_slash=False)
router.register('image',ImageView,base_name='image')
router.register('test-model',TestModelView,base_name='test'),
router.register('options',OptionsView,base_name='options'),

urlpatterns = router.urls
# urlpatterns = [
#     url(r'^',include(router.urls)),
# ]



import setuptools
with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="meapp",
    version="0.0.14",
    author="zhuxietong",
    author_email="zhuxietong@me.com",
    description="A small example packcage add tools",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/example-project",
    packages=setuptools.find_packages(),
    # packages=setuptools.find_packages(include=['core', 'location', 'tools','api']),
    classifiers=(
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ),
    install_requires=[
        'django',
        'djangorestframework',
        'six',
        'Pillow',
        'aliyun-python-sdk-core-v3',
        'aliyun-python-sdk-ecs',
        # 'mysqlclient'
    ],
    include_package_data=True,
)



__author__ = 'zhuxietong'
from rest_framework.views import exception_handler
from rest_framework.response import Response
from rest_framework.exceptions import APIException
from .response import ApiState
from .crossdomain import crossdomain
import json


from django.conf import settings

ApiCode = 'status'
ApiData = 'data'
ApiMessage = 'message'

try:
    if settings.ApiCode is not None:
        ApiCode = settings.ApiCode
except Exception as e:
    pass
try:
    if settings.ApiData is not None:
        ApiData = settings.ApiData
except Exception as e:
    pass

try:
    if settings.ApiMessage is not None:
        ApiMessage = settings.ApiMessage
except Exception as e:
    pass



class EeException(APIException):
    def __init__(self, instance,line=None,file=None):
        super(EeException,self).__init__(instance)


def paw_message(data):
    if isinstance(data, str):
        detail = data
    elif isinstance(data, dict):
        infos = []
        print(data)
        for key, value in data.items():
            msg = paw_message(value)
            one_message = "%s:%s" % (key,msg)
            print(one_message)
            infos.append(one_message)
        detail = ';'.join(infos)
    elif isinstance(data,list):
        infos = []
        for value in data:
            one_msg = paw_message(value)
            infos.append(one_msg)
        detail = ';'.join(infos)
    else:
        detail = json.dumps(data)
    return  detail


@crossdomain('')
def custom_exception_handler(exc, context):
    # Call REST framework's default exception handler first,
    # to get the standard error response.
    # print("===DD",type(exc))
    response = exception_handler(exc, context)
    # Now add the HTTP status code to the response.
    if response is not None:
        message = paw_message(response.data)
        data = {
            ApiCode:ApiState.exception.value,
            ApiData:{},
            ApiMessage:message
        }
        response.data = data
        response.status_code = 200

    else:
        data = {}
        data[ApiCode] = ApiState.exception.value
        data[ApiData] = None
        data[ApiMessage] = str(exc)
        response = Response(data=data, status=200, headers=None)
        response.status_code = 200
    return response



from django.db import models

# Create your models here.
from django.db import models

import uuid

# -*- coding: UTF-8 -*-
from django.core.files.storage import FileSystemStorage


class ImageStorage(FileSystemStorage):
    from django.conf import settings

    def __init__(self, location=settings.MEDIA_ROOT, base_url=settings.MEDIA_URL):
        # 初始化
        super(ImageStorage, self).__init__(location, base_url)

    # 重写 _save方法
    def _save(self, name, content):
        import os, time, random
        # 文件扩展名
        ext = os.path.splitext(name)[1]
        # 文件目录
        d = os.path.dirname(name)
        # 定义文件名，年月日时分秒随机数
        fn = time.strftime('%Y%m%d%H%M%S')
        fn = fn + '_%d' % random.randint(0, 100)
        # 重写合成文件名
        name = os.path.join(d, fn + ext)
        # 调用父类方法
        return super(ImageStorage, self)._save(name, content)
#
# try:
#     from eeapp.eeapp.model.Images import ImageModel
# except:
#     from ...eeapp.eeapp.model.Images import ImageModel


from django.conf import settings
Default_IMAGE = '%sfine_uploader/head1.jpg' % settings.STATIC_URL

from django.conf import settings
HOST = settings.MEDIA_HOST
class BaseImageModel(models.Model):
    @classmethod
    def default_url(cls):
        return Default_IMAGE

    filename = models.CharField(max_length=100,verbose_name='文件名')
    image = models.ImageField(upload_to='img/%Y/%m/%d',storage=ImageStorage())


    def _url(self):
        try:
            if self.image.url:
                return HOST + self.image.url
            return Default_IMAGE
        except Exception as e:
            return self.image.url
    url = property(_url)

    class Meta:
        ordering = ['id']
        abstract = True


    def save(self, force_insert=False, force_update=False, using=None,
             update_fields=None):

        iuuid = uuid.uuid4()
        self.filename = iuuid
        super(BaseImageModel,self).save(force_insert=force_insert,force_update=force_update,using=using,update_fields=update_fields)






class ImageModel(BaseImageModel):
    class Meta:
        ordering = ['id']
        abstract = False
